%% LINEARIZATION OF FULL DSM SYSTEM

close all;
clear all;
clc;

%% PARAMETERS
%% On-board system
Sn_os = 12.8e6;                   % Rated power of the subsea system
Vn_llrms_os = 11e3;             % Rated voltage, nominal rms line-line voltage [V] (NORSOK, when total installed capacity exceeds 20MW)
Vn_os = sqrt(2)/sqrt(3)*Vn_llrms_os; % (V) Rated voltage, nominal peak phase-ground voltage [V]
In_os = 2/3*Sn_os/Vn_os;        % (A) Rated peak phase current
Zn_os = Vn_os/In_os;            % (Ohm) Rated impedance ac
fn_os =  50;                    % Rated grid frequency
wn_os = 2*pi*fn_os;

%% Topside VSC values (FROM LECTURE ELK23)

lc_pu = 0.15;                   % PU value of the converter reactor
rc_pu = 0.01;                   % PU value of the converter resistance 
cc_dcpu = 0.88;                 % PU value of capacitor on the DC side

Req_pu = rc_pu;                 % Equivalent resistance (on-bord + VSC resistance)
Leq_pu = lc_pu;                 % Equivalent inductance (on-bord + VSC reactance)

Fsw_c = 10000;                  % Switching freqency
Tsw_c = 1/Fsw_c;                % Switching period 
Ta = Tsw_c/2;                   % Average PWM delay 
tau_e = 1.5*Tsw_c;              % Time delay including computation time and A/D D/A conversion. 
tau_eq = 2*tau_e;               % First order equivalent (VCC) time constant

%% VCC tuning

tau_cc = lc_pu/(wn_os*rc_pu);    % Time constant
Kpc_c = tau_cc*rc_pu/(2*tau_e);  % Current proportional gain
Kic_c = Kpc_c/(tau_cc);          % Current integral gain

%% DVC tuning
alpha = sqrt(6);                        % Selected with regards to phase margin

Kp_v = tau_e/(tau_eq*alpha); 
%Kp_v = 1/(wn_os*cc_dcpu*tau_eq*alpha);  % Voltage proportional gain
Tv_c = alpha^2*tau_eq;                  % Voltage PI time constant
Ki_v = Kp_v/(Tv_c);                  % Voltage integral gain

%% DC base values
Vn_dc = sqrt(6)*Vn_os;          % (V) Base dc voltage
In_dc = Sn_os/Vn_dc;            % (I) Base DC current
Zn_dc = Vn_dc/In_dc;            % (Ohm) Base DC impedance

%% Cable (pi-equivalent) (2XS(FL)2YRAA 3x1x150 RM/25 12/20 (24)kV)

lenght = 3;                     % Length of cable in km

R_link = 0.06;                  % Ohms/km
L_link = 0.327e-3;              % H/km
C_link = 0.355e-6;              % F/km

rl = R_link*lenght/Zn_dc;             % Cable resistance in pu
ll = L_link*lenght*wn_os/Zn_dc;       % Cable inductance in pu
cl = 1/(C_link*lenght*wn_os*Zn_dc);   % Cable capacitance in pu

Ceq_pu = cl + cc_dcpu;            % Equivalent capacitance (VSC+link)

%% Subsea system

% Rated values load
Sn_ss = 7.8e6;                        % (VA) Base 3-phase apparent power 
Vn_llrms_ss = 6e3;                    % (v) Base voltage, line-to-line rms
Vn_ss = Vn_llrms_ss*sqrt(2)/sqrt(3);  % (V) Base voltage, nominal peak phase-ground voltage [V]
In_ss = 2/3*Sn_ss/Vn_ss;              % (A) Base peak phase current
Zn_ss = Vn_ss/In_ss;                  % (Ohm) Base impedance
fn_ss = 50;                           % (Hz) Rated frequenzy
wn_ss = 2*pi*fn_ss;

% VSD
Fsw_d = 5000;                   % Switching freqency
Tsw_d = 1/Fsw_d;                % Period 
T_delay = Tsw_d/2;              % Avergae PWM delay 
T_eq =1.5*T_delay;                % First order equivalent (VCC) time constant 

% Induction machine (SI) (ABB 9MW machine)

p = 2;                           % Number of polespairs 
Rs = 0.01887;                   % (Ohm) Stator resitance
Xs_leak = 0.6271;               % (OHm) Stator leakage reactance
Rr = 0.01866;                   % (Ohm) Rotor resistance
Xr_leak = 0.3748;               % (Ohm) Rotor leakage reactance
Xm = 24.82;                     % (Ohm) Magnetizing inductance     

Xs = Xm+Xs_leak;    
Xr = Xm+Xr_leak;    
     
% Mechanical system   

n_mech_n = wn_ss/p;           % (rad/s) Synchronous mechanical speed
Torque_n = Sn_ss/n_mech_n;    % Nominal Torque 
Torque = 57607;                 % Rated torque ABB machine (Nm)
Torque_pu = Torque/Torque_n;    % Torque pu
N_n_rpm = 1492;                 % Rated speed (rpm)
n_mech = N_n_rpm*(2*pi/(60*wn_ss)); % Angular velocity of shaft (pu)  

Jload = 0;                      % Load inertia
Jm = 556;                       % (kgm2) Rotor inertia
Jtot = Jload + Jm;              % Total inertia
Max_torque_pu = Torque_pu;      % Max torque
Km = Max_torque_pu;             % Machine constant          

Tm = Jtot*n_mech_n^2/Sn_ss;   % Mechanical time constant

% Induction machine per unit calculations

rs_pu = Rs/Zn_ss;                  % (pu) Stator resitance
xs_pu = Xs/Zn_ss;                  % (pu) Stator reactance 

rr_pu = Rr/Zn_ss;                  % (pu) Rotor resistance
xr_pu = Xr/Zn_ss;                  % (pu) Rotor reactance 
xm_pu = Xm/Zn_ss;                  % (pu) Magentizing inductance                

% Aggregated constants

r_R   = (xm_pu/xr_pu)^2*rr_pu;
rs_   = rs_pu+r_R;
Tr    = xr_pu/(wn_ss*rr_pu);
sigma = 1-xm_pu^2/(xs_pu*xr_pu);
x_sigma = sigma*xs_pu;
x_M   = xm_pu^2/xr_pu;
Ts_   = x_sigma/(wn_ss*rs_);

%% Tuning of VSD
beta = 4;                           

% VCC
Kpc_m = (x_sigma)/(2*wn_ss*T_delay);
Tic = Ts_; 
Kic_m = Kpc_m/Tic;

% Speed controller
Kp_sp = Tm/(sqrt(beta)*T_eq);
Ti_sp =beta*T_eq;
Ki_sp = Kp_sp/Ti_sp;

%% Steady state conditions

id1 =  0.8;                  % PU value of the d component of the ac grid side current at operating point
iq1 =  0;                  % PU value of the q component of the ac grid side current at operating point
vgd =  1;                  % PU value of the d component of the ac grid voltage at operating point
vgq =  0;                  % PU value of the q component of the ac grid voltage at operating point
wos =  1;                  % PU value of the the electrical speed on board

id2  = 0.4;                % (pf = 0.9) PU value of the q component of the ac load side current at operating point
iq2  = 0.7;                % PU value of the q component of the ac load side current at operating point

wss  = 1;                  % PU value of the the electrical speed on subsea
psiR = 2.5;                % PU value of the rotor flux

vdc1 = 1;                  % PU value for dc voltage at VSC1 at operating point
vdc2 = 1;                  % PU value for dc voltage at VSC2 at operating point
i12  = 0.8;                % PU value of the  cable current at operating point

epsd1 = 0;                 % PU value of the integral of the error of the d current at VSC1
epsq1 = 0;                 % PU value of the integral of the error of the q current at VSC1
epsd2 = 0;                 % PU value of the integral of the error of the q current at VSC2
epsq2 = 0;                 % PU value of the integral of the error of the q current at VSC2
epsv  = 0;                 % PU value of the integral of the dc voltage error
epssp = 0;                 % PU value of the integral of the speed error 

Vdc_ref = 1;               % PU value for dc voltage reference for VSC
Q_ref   = 0;               % PU value for reactive power reference for VSC
n_ref   = n_mech;          % PU value for active power reference for VSC2
i_psi_R_ref = 0.4;         % PU value for magnetizing current in drive


%% LINEARIZATION

% Model parameters
syms Req_pu_ Leq_pu_ lc_pu_ Kpc_c_ Kic_c_ Kp_v_ Ki_v_ Vdc_ref_ Q_ref_ ...                 % On-board system + VSC
     Ceq_pu_ rl_ ll_      ...                                                             % DC link
     x_sigma_ rs__ Tr_ x_M_ r_R_ Kpc_m_ Kic_m_ Kp_sp_ Ki_sp_ Tm_ Km_ i_psi_R_ref_ n_ref_ p_;    % Drive + motor

% Model variables
syms i_d1 i_q1 vc_d vc_q vg_d vg_q w_os eps_d1 eps_q1 eps_v ...               % On-board system + VSC
     v_dc1 v_dc2 i_1 i_2 i_12 ...                                                               % DC-lnk
     i_d2 i_q2 vm_d vm_q psi_R n_mech_ fs_ w_ss eps_d2 eps_q2 eps_sp ;                             % Drive + motor

%% System equations dx/dt = f(x,t)

% On-board system

% DVC
% Integral term DVC (Ek=d/dt(eps_v))
Ev = Vdc_ref_-v_dc1;

id1_ref = Kp_v_*(Vdc_ref_-v_dc1)+Ki_v_*eps_v+i_12;
iq1_ref=  Q_ref_/vg_d;

% VCC_C
% Integral term VCC1 (Ek=d/dt(eps_i_dq))
Ec11 =  id1_ref-i_d1;
Ec12 =  iq1_ref-i_q1;

vc_d = -Kpc_c_*(id1_ref-i_d1)-Kic_c_*eps_d1+w_os*lc_pu_*i_q1+vg_d;
vc_q = -Kpc_c_*(iq1_ref-i_q1)-Kic_c_*eps_q1-w_os*lc_pu_*i_d1+vg_q;

% AC topside
F11 = -(Req_pu_/Leq_pu_)*i_d1+w_os*i_q1+vg_d/Leq_pu_-vc_d/Leq_pu_;
F12 = -(Req_pu_/Leq_pu_)*i_q1-w_os*i_d1+vg_q/Leq_pu_-vc_q/Leq_pu_;

% SUBSEA

% VSD

% Speed controller
Es = n_ref_-n_mech_; % Integral term (Ek=d/dt(eps_sp))
iq2_ref = (Kp_sp_*(n_ref_-n_mech_)+Ki_sp_*eps_sp)/psi_R;

% Flux reference
id2_ref = i_psi_R_ref_;

% Frequency of the rortor flux
f_psi_r_ = r_R_*i_q2/psi_R+n_mech_*p_;

% Integral term VCC_d (Ek=d/dt(eps_i_dq))
Ec21 = id2_ref-i_d2;
Ec22 = iq2_ref-i_q2;

vm_d = Kpc_m_*(id2_ref-i_d2)+Kic_m_*eps_d2-f_psi_r_*x_sigma_*i_q2-psi_R/(w_ss*Tr_);
vm_q = Kpc_m_*(iq2_ref-i_q2)+Kic_m_*eps_q2+f_psi_r_*x_sigma_*i_d2+n_mech_*p_*psi_R;

% Rotor oriented model 
F21 = (w_ss/x_sigma_)*(-rs__*i_d2+f_psi_r_*x_sigma_*i_q2+psi_R/(w_ss*Tr_)+vm_d);
F22 = (w_ss/x_sigma_)*(-rs__*i_q2-f_psi_r_*x_sigma_*i_d2-n_mech_*p_*psi_R+vm_q);

P11 = -1/Tr_*psi_R+x_M_/Tr_*i_d2;


% Mechanical system
tau_e = psi_R*i_q2;
tau_l = Km_*n_mech_^2;

M = 1/Tm_*(tau_e-tau_l); 

% Power balance                             (i_dc*v_dc = i_d*iv_d+i_q*v_q)
i_1 = (vc_d*i_d1+i_q1*vc_q)/v_dc1;
i_2 = (vm_d*i_d2+i_q2*vm_q)/v_dc2;

% DC link
G1 = 1/Ceq_pu_*(i_1 - i_12);                                % d/dt(v_dc1)
G2 = 1/Ceq_pu_*(i_12 - i_2);                                % d/dt(v_dc2)
G3 = -(rl_/ll_)*i_12 + (1/ll_)*v_dc1 - (1/ll_)*v_dc2;       % d/dt(i_12)

%% STATE MATRIX

F  =    [ F11;  F12;   Ec11;   Ec12;    Ev;  F21;  F22;   P11;     Es;   Ec21;   Ec22;       M;    G1;    G2;   G3];
xs =    [i_d1; i_q1; eps_d1; eps_q1; eps_v; i_d2; i_q2; psi_R; eps_sp; eps_d2; eps_q2; n_mech_; v_dc1; v_dc2; i_12];


% Compute Jacobian symbolically
A_symbolic = jacobian(F,xs);
% A_simp = simplify(A_symbolic)

%Compute the state space matrices algebraically

% Kpc_c = 1;
% Kp_v = 10;
% Kpc_m = 5;
% Kp_sp = 4;
% 
% Kic_c = 2;
% Ki_v = 0.7;
% Kic_m = 20;
% Ki_sp = 0.3;
%------------ 
% %Base case
% Kpc_c = 1;
% Kp_v = 3;
% Kpc_m = 1.5;
% Kp_sp = 1.5;
% 
% Kic_c = 3;
% Ki_v = 0.5;
% Kic_m = 3.5;
% Ki_sp = 2.5;

% Official SIMULINK
Kpc_c = 0.2;
Kp_v = 1;
Kpc_m = 0.3;
Kp_sp = 1;

Kic_c = 1;
Ki_v = 4;
Kic_m = 6;
Ki_sp = 8;

% -------------------------------------------------------------------------                                    
%Kpc_c = 0.005;
%Kic_c = 1.01;
%Kp_v = 0.5;
%Ki_v = 1.5;
%Kpc_m = 0.007;
%Kic_m = 2;
%Kp_sp = 0.03;
%Ki_sp = 0.1;

X_algebraic = simplify(subs(A_symbolic,{Req_pu_ Leq_pu_ lc_pu_  Kpc_c_  Kic_c_  Kp_v_ Ki_v_ Vdc_ref_ Q_ref_ Ceq_pu_ rl_ ll_ x_sigma_ rs__ Tr_ x_M_ r_R_ Kpc_m_ Kic_m_ Kp_sp_ Ki_sp_ Tm_ Km_  i_psi_R_ref_ n_ref_ i_d1 i_q1 vg_d vg_q  w_os  eps_d1 eps_q1  eps_v v_dc1 v_dc2 i_12 i_d2 i_q2 psi_R n_mech_ w_ss eps_d2 eps_q2 eps_sp p_},...
                                         [Req_pu Leq_pu  lc_pu   Kpc_c   Kic_c   Kp_v  Ki_v  Vdc_ref  Q_ref  Ceq_pu  rl  ll  x_sigma  rs_  Tr  x_M  r_R  Kpc_m  Kic_m  Kp_sp  Ki_sp  Tm  Km   i_psi_R_ref  n_ref  id1  iq1  vgd  vgq  wos    epsd1  epsq1   epsv  vdc1  vdc2  i12  id2  iq2  psiR  n_mech  wss  epsd2   epsq2  epssp p]));                                


X_eval = eval(X_algebraic);
eop = eig(X_eval);

XRratio = 1;
lth = 0.005;                 % Thevenin inductance (on-bord)
rth = lth*XRratio;                 % Thevenin resistance (on-bord)
SCR1 = 1/sqrt(lth^2+rth^2);

Req_pu = rc_pu+rth;          % Equivalent resistance (on-bord + VSC resistance)
Leq_pu = lc_pu+lth;          % Equivalent inductance (on-bord + VSC reactance)


S1_algebraic = simplify(subs(A_symbolic,{Req_pu_ Leq_pu_ lc_pu_  Kpc_c_  Kic_c_  Kp_v_ Ki_v_ Vdc_ref_ Q_ref_ Ceq_pu_ rl_ ll_ x_sigma_ rs__ Tr_ x_M_ r_R_ Kpc_m_ Kic_m_ Kp_sp_ Ki_sp_ Tm_ Km_  i_psi_R_ref_ n_ref_ i_d1 i_q1 vg_d vg_q  w_os  eps_d1 eps_q1  eps_v v_dc1 v_dc2 i_12 i_d2 i_q2 psi_R n_mech_ w_ss eps_d2 eps_q2 eps_sp p_},...
                                         [Req_pu Leq_pu  lc_pu   Kpc_c   Kic_c   Kp_v  Ki_v  Vdc_ref  Q_ref  Ceq_pu  rl  ll  x_sigma  rs_  Tr  x_M  r_R  Kpc_m  Kic_m  Kp_sp  Ki_sp  Tm  Km   i_psi_R_ref  n_ref  id1  iq1  vgd  vgq  wos    epsd1  epsq1   epsv  vdc1  vdc2  i12  id2  iq2  psiR  n_mech  wss  epsd2   epsq2  epssp p]));                                

                                     
S1_eval = eval(S1_algebraic);
eop1 = eig(S1_eval)

[P1, D1]=eig(S1_eval);
QT1=inv(P1);
Q1=QT1';
participationF1 = abs(Q1.*P1);                               
% -------------------------------------------------------------------------                                    
% Kpc_c = 1.5;
% Kic_c = 4;
% Kp_v = 450;
% Ki_v = 599.5;
%Kpc_m = 2;
%Kic_m = 8;
%Kp_sp = 9;
%Ki_sp = 30;

XRratio =1;
lth = 0.05*300;                 % Thevenin inductance (on-bord)
rth = lth*XRratio;                 % Thevenin resistance (on-bord)
SCR2 = 1/sqrt(lth^2+rth^2);

Req_pu = rc_pu+rth;          % Equivalent resistance (on-bord + VSC resistance)
Leq_pu = lc_pu+lth;          % Equivalent inductance (on-bord + VSC reactance)


S2_algebraic = simplify(subs(A_symbolic,{Req_pu_ Leq_pu_ lc_pu_  Kpc_c_  Kic_c_  Kp_v_ Ki_v_ Vdc_ref_ Q_ref_ Ceq_pu_ rl_ ll_ x_sigma_ rs__ Tr_ x_M_ r_R_ Kpc_m_ Kic_m_ Kp_sp_ Ki_sp_ Tm_ Km_  i_psi_R_ref_ n_ref_ i_d1 i_q1 vg_d vg_q  w_os  eps_d1 eps_q1  eps_v v_dc1 v_dc2 i_12 i_d2 i_q2 psi_R n_mech_ w_ss eps_d2 eps_q2 eps_sp p_},...
                                        [Req_pu Leq_pu  lc_pu   Kpc_c   Kic_c   Kp_v  Ki_v  Vdc_ref  Q_ref  Ceq_pu  rl  ll  x_sigma  rs_  Tr  x_M  r_R  Kpc_m  Kic_m  Kp_sp  Ki_sp  Tm  Km   i_psi_R_ref  n_ref  id1  iq1  vgd  vgq  wos    epsd1  epsq1   epsv  vdc1  vdc2  i12  id2  iq2  psiR  n_mech  wss  epsd2   epsq2  epssp p]));
                                                                                                
S2_eval = eval(S2_algebraic);
eop2 = eig(S2_eval)

[P2, D2]=eig(S2_eval);
QT2=inv(P2);
Q2=QT2';
participationF2 = abs(Q2.*P2);

%% Plotting

x1 = linspace(0,-2);
y1 = sqrt(2)/2*(-x1);

x2 = linspace(0,-2);
y2 = sqrt(2)/2*(x2);


%% Sensitivity analysis

V = zeros(200,15);
damping = zeros(15,1);
dampingx = zeros(15,1);

cc = jet(300);
 
for j=1:300

XRratio = 0.2;     
lth = 0.005*j;            % Thevenin inductance (on-bord)
rth = lth*XRratio;          % Thevenin resistance (on-bord)
zth = sqrt(lth^2+rth^2);

Req_pu = rc_pu+rth;          % Equivalent resistance (on-bord + VSC resistance)
Leq_pu = lc_pu+lth;          % Equivalent inductance (on-bord + VSC reactance)
    

S_algebraic = simplify(subs(A_symbolic,{Req_pu_ Leq_pu_ lc_pu_  Kpc_c_  Kic_c_  Kp_v_ Ki_v_ Vdc_ref_ Q_ref_ Ceq_pu_ rl_ ll_ x_sigma_ rs__ Tr_ x_M_ r_R_ Kpc_m_ Kic_m_ Kp_sp_ Ki_sp_ Tm_ Km_  i_psi_R_ref_ n_ref_ i_d1 i_q1 vg_d vg_q  w_os  eps_d1 eps_q1  eps_v v_dc1 v_dc2 i_12 i_d2 i_q2 psi_R n_mech_ w_ss eps_d2 eps_q2 eps_sp p_},...
                                        [Req_pu Leq_pu  lc_pu   Kpc_c   Kic_c   Kp_v  Ki_v  Vdc_ref  Q_ref  Ceq_pu  rl  ll  x_sigma  rs_  Tr  x_M  r_R  Kpc_m  Kic_m  Kp_sp  Ki_sp  Tm  Km   i_psi_R_ref  n_ref  id1  iq1  vgd  vgq   wos   epsd1  epsq1   epsv  vdc1  vdc2  i12  id2  iq2  psiR  n_mech  wss  epsd2  epsq2  epssp p]));
                                    
S_eval = eval(S_algebraic);
                                    
ei = eig(S_eval);

for i = 1:15
    damping1 = -real(ei(i))/((imag(ei(i))^2+real(ei(i))^2))^0.5;
    damping(j,i) = damping1;
    V(j,i) = ei(i);

end

figure(2)
     hold on
     grid on;
     line(x1,y1,'Color','black','LineStyle','--');
     line(x2,y2,'Color','black','LineStyle','--');
     scatter(real(eop),imag(eop),200,'k','x'); 
     scatter(real(ei),imag(ei),[], cc(j,:)) %   Plot real and imaginary parts
     xlabel('Real','FontSize',15)
     ylabel('Imaginary','FontSize',15)
     t1 = 'Eigenvalues of a closed loop system';
     hold off;
     set(gca,'FontSize',15);
end

%% Computation for single case


%  A_eval = eval(S1_algebraic);
%  eop = eig(A_eval)

% [P, D]=eig(A_eval);
% QT=inv(P);
% Q=QT';
% participationF = abs(Q.*P);

%  eig_mat = zeros(15,2);
%  pf_tab = zeros(15,15);
  pf_mat_mm = zeros(15,30);

    for i = 1:15
      
%    eig_mat(i,1) = round(real(eop(i)*1000))/1000;
%    eig_mat(i,2) = round(imag(eop(i)*1000))/1000;
       for j = 1:15
%     damping1 = -real(eop(i))/((imag(eop(i))^2+real(eop(i))^2))^0.5;
%     dampingx(i,1) = damping1;
 
%    damping(i,1) = round(damping1*100)/100;
%       pf_tab(i,j)= (round(participationF(i,j)*1000)/1000);
       pf_tab_mm(i,j)= table(round(participationF1(i,j)*1000)/1000);
      end
%      
      for h = 16:30
       pf_tab_mm(i,h)= table(round(participationF2(i,h-15)*1000)/1000);
      end
 
   end

%eig_tab = table(eig_mat(:,1),eig_mat(:,2),damping);
%PF_tab = table(pf_tab);
figure(1)
     scatter(real(eop1),imag(eop1),[]);
     hold on
     scatter(real(eop2),imag(eop2),[]); 
%    scatter(real(eop),imag(eop),[]); 

     grid on;     
     line(x1,y1,'Color','black','LineStyle','--');
     line(x2,y2,'Color','black','LineStyle','--');
     xlabel('Real')
     ylabel('Imaginary')
     t1 = 'Eigenvalues of a closed loop system';
     title('Eigenvalue plot (open loop)'); 
     hold off
       
%writetable(eig_tab,'Macintosh HD\Brukere\josteingjevre\Dokumenter\NTNU\Thesis\tables\eigenvalues.csv');
%writetable(PF_tab,'participation_fac_init.csv');
%writetable(pf_tab_mm,'participation_fac_init.csv');