% SYSTEM INITIALIZATION

close all;
clear all;
clc;

Tsamp = 20e-6;                  % Sampling time in simulation  

% Basic VSC system parameters (from lecture ELK23)

Lc_pu = 0.15;                   % PU value of the line inductor
Rc_pu = 0.01;                   % PU value of the line resistance 
C_dcpu = 0.88;                  % PU value of capacitor on the DC side
lth = 0.14;                     % 
rth = 0.14;                 % 
Fsw_c = 10000;                  % Switching freqency
fn_os = 50;                     % Grid frequency     
wn_os  = 2*pi*fn_os;            % Angular velocity
% Time constants converter    (From lecture ELK23)

T_sw_c = 1/Fsw_c;                  % Period 
Tpwm = T_sw_c/2;                   % Avergae PWM delay 
tau_e = 1.5*T_sw_c;                % Time delay including computation time and A/D D/A conversion. 
tau_eq = 2*tau_e;                  % First order equivalent (VCC) time constant 

% Base values ac on-board system

Sn_os = 12.8e6;                    % (VA)Base 3phase apparent power (max contemporary power) 
Vn_ll_os = 11e3;                   % (V) Base voltage, nominal rms line-line voltage [V] (NORSOK standard)
Vn_os = Vn_ll_os*sqrt(2)/sqrt(3);  % (V) Base voltage, nominal peak phase-ground voltage [V]
In_os = 2/3*Sn_os/Vn_os;           % (A) Base peak phase current
Zn_os = Vn_os/In_os;               % (Ohm) Base impedance ac

% Base values dc

Vdcb = sqrt(6)*Vn_os;              % (V) Base dc voltage (three core bipolar)
Idcb = Sn_os/Vdcb;                 % (I) Base DC current
Zdcb = Vdcb/Idcb;                  % (Ohm) Base DC impedance

% Physical values converter

Lc = Lc_pu*Zn_os/wn_os;            % (H) Physical value of the inductance
Rc = Rc_pu*Zn_os;                  % (Ohm) Physical value of the resistance
Cc = 1/(wn_os*C_dcpu*Zdcb);        % (F) Physical converter capacitance

% Physical values of thevenin impedance

Lth = lth*Zn_os/wn_os;            % (H) Physical value of the inductance
Rth = rth*Zn_os;                  % (Ohm) Physical value of the resistance

%% Cable parameters 2XS(FL)2YRAA 3x1x150 RM/25 12/20 (24)kV

km = 3;                         % Length of cable in km
Rdcpu = 0.06;                   % Ohms/km/conductor
Ldcpu = 0.327e-3;               % H/km/conductor
Ccpu = 0.355e-6;                % F/km/conductor
Rl = Rdcpu*km;                  % Cable resistance pr. conductor
Ll = Ldcpu*km;                  % Cable inductance pr. conductor
Cl = Ccpu*km;                   % Cable capacitance pr.conductor

%% VCC tuning

tau =  Lc_pu/(wn_os*Rc_pu);  % Time constant
Kpc =  tau*Rc_pu/(2*tau_e)/1000;  % Current proportional gain
Kic =  0; %Kpc/tau;              % Current integral gain

%% Tuning DVC

alpha = sqrt(6);                        % Selected with regards to phase margin
Kp_v_tp_x = tau_e/(tau_eq*alpha);              
Kp_v_tp = 1/(wn_os*C_dcpu*tau_eq*alpha); % Voltage proportional gain
Tv = alpha^2*tau_eq;                    % Voltage PI time constant
Ki_v_tp = Kp_v_tp_x/(Tv);                      % Voltage integral gain

%% SUBSEA SYSTEM   

% Necessary to make the model run (Have no impact on the results)
%-------------------------------------------------------------------------
Lf = 1e-6;               % (H) Physical value of the inductance
Rf = 1e-9;               % (Ohm) Physical value of the resistance
Cf = 1e-9;               % (F) Physical value of the capacitance
tau_sp_ref = 1e-3;       % Rate limiter for speed reference
%-------------------------------------------------------------------------
Cdc = Cc;                    % (F) Physical DC capacitance
F_sw = 5000;                 % Swiching freq. (based on typical GPD freq.)
T_sw_c = 1/Fsw_c;            % Switching period 
T_delay = T_sw_c/2;          % Avergae PWM delay 
T_eq =1.5*T_delay;           % First order equivalent (VCC) time constant 
T_sum = T_eq;                % Considered to be the same as T_eq

% Base values ac

fn_ss = 50;                             % (Hz) Rated frequency
wn_ss = 2*pi*fn_ss;                        % (rad/s) Rated angular velocity
Sn_ss = 7.8e6;                          % (VA) Base 3phase apparent power 
Vn_llrms_ss = 6e3;                      % (v) Base voltage, line-to-line rms
Vn_ss = Vn_llrms_ss*sqrt(2)/sqrt(3);    % (V) Base voltage, nominal peak phase-ground voltage [V]
In_ss = 2/3*Sn_ss/Vn_ss;                % (A) Base peak phase current
Zn_ss = Vn_ss/In_ss;                    % (Ohm) Base impedance ac

%% Machine parameters (SI) (9 MW ABB machine)
 
Rs = 0.01887;                   % (Ohm) Stator resitance
Xs_leak = 0.6271;               % (OHm) Stator leakage reactance
Rr = 0.01866;                   % (Ohm) Rotor resistance
Xr_leak = 0.3748;               % (Ohm) Rotor leakage reactance
Xm = 24.82;                     % (Ohm) Magnetizing inductance     

Xs = Xm+Xs_leak;    
Xr = Xm+Xr_leak;    


Torque_n = 57607*(Sn_ss/9e6);   % Rated torque (Nm)
N_n_rpm = 1492;                 % Rated speed (rpm)
n_n = N_n_rpm*(2*pi/60);        % Rated speed (rad/sec)   
p = 2;                          %  Number of polespairs 
Jload = 0;                      % Load inertia
Jm = 556;                       % Rotor inertia
Jtot = Jload + Jm;              % Total inertia
Max_torque = Torque_n;          % Max torque
Km = Max_torque/n_n^2;          % Machine constant             
N_ref = N_n_rpm;                % Speed reference

% Machine parameters (pu)

r_s = Rs/Zn_ss;                    % (pu) Stator resitance
x_s = Xs/Zn_ss;                    % (pu) Stator reactance
r_r = Rr/Zn_ss;                    % (pu) Rotor resistance
x_r = Xr/Zn_ss;                    % (pu) Rotor reactance
x_m = Xm/Zn_ss;                     % (pu) Mutual reactance

%% Aggregated constants (from drive compendium)

r_R = (x_m/x_r)^2*r_r;
rs_ = r_s+r_R;
Tr = x_r/(wn_ss*r_r);
sigma = 1-x_m^2/(x_s*x_r);
x_sigma = sigma*x_s;
x_M = x_m^2/x_r;
Ts_ = x_sigma/(wn_ss*rs_);

psi_R_init = 0.01;                     %Initializing condition

% Mech constants
Omega_base = wn_ss/p;                  % (1/s) synchronous mechanical speed
Tm = Jtot*Omega_base^2/Sn_ss;          % Mechanical time constant
Max_load = 40;
time=10;

%% Tuning of controller (from drive compendium)
beta = 4;

% VCC
Kpc_m = 0.2; %3.6/3; %(x_sigma)/(2*wn_ss*T_delay)/500;
Tic = Ts_; 
Kic_m = 4; %16/15; %Kpc_m/Tic;

I_d_ref = 0.4;

% Speed controller
Kp_sp = 40; %12; %Tm/(sqrt(beta)*T_sum)/3000;
Ti_sp =beta*T_sum;
Ki_sp = 10; % 150/10; %Kp_sp/Ti_sp/2000;

%% DO NOT TOUCH - WORKING FOR SEPARATE MODEL

Kpc = 0.2;
Kp_v = 0.25;
Kpc_m = 0.2;
Kp_sp = 15;

Kic = 1;
Ki_v = 4;
Kic_m = 4;
Ki_sp = 5;

%% Test 


Kpc = 0.2;
Kp_v = 0.25;
Kpc_m = 0.3;
Kp_sp = 20;

Kic = 1;
Ki_v = 4;
Kic_m = 6;
Ki_sp = 8;

speed_init = 0;

% Official

Kpc = 0.2;
Kp_v = 0.5;
Kpc_m = 0.3;
Kp_sp = 1;

Kic = 1;
Ki_v = 4;
Kic_m = 6;
Ki_sp = 8;


% Official test

% Kpc = 0.2;
% Kp_v = 0.5;
% Kpc_m = 0.3;
% Kp_sp = 1;
% 
% Kic = 1;
% Ki_v = 4;
% Kic_m = 6;
% Ki_sp = 8;
